/****************************************************************************
**
** Copyright (C) 1992-2004 Trolltech AS. All rights reserved.
**
** This file is part of the compatibility classes of the Qt Toolkit.
**
** Licensees holding valid Qt Preview licenses may use this file in
** accordance with the Qt Preview License Agreement provided with the
** Software.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
** information about Qt Commercial License Agreements.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef Q3URL_H
#define Q3URL_H

#include "qstring.h"

#ifndef QT_NO_URL

class Q3UrlPrivate;

class Q_COMPAT_EXPORT Q3Url
{
public:
    Q3Url();
    Q3Url(const QString& url);
    Q3Url(const Q3Url& url);
    Q3Url(const Q3Url& url, const QString& relUrl, bool checkSlash = false);
    virtual ~Q3Url();

    QString protocol() const;
    virtual void setProtocol(const QString& protocol);

    QString user() const;
    virtual void setUser(const QString& user);
    bool hasUser() const;

    QString password() const;
    virtual void setPassword(const QString& pass);
    bool hasPassword() const;

    QString host() const;
    virtual void setHost(const QString& user);
    bool hasHost() const;

    int port() const;
    virtual void setPort(int port);
    bool hasPort() const;

    QString path(bool correct = true) const;
    virtual void setPath(const QString& path);
    bool hasPath() const;

    virtual void setEncodedPathAndQuery(const QString& enc);
    QString encodedPathAndQuery();

    virtual void setQuery(const QString& txt);
    QString query() const;

    QString ref() const;
    virtual void setRef(const QString& txt);
    bool hasRef() const;

    bool isValid() const;
    bool isLocalFile() const;

    virtual void addPath(const QString& path);
    virtual void setFileName(const QString& txt);

    QString fileName() const;
    QString dirPath() const;

    Q3Url& operator=(const Q3Url& url);
    Q3Url& operator=(const QString& url);

    bool operator==(const Q3Url& url) const;
    bool operator==(const QString& url) const;

    inline bool operator!=(const Q3Url &url) const { return !(operator==(url)); }
    inline bool operator!=(const QString &url) const { return !(operator==(url)); }

    static void decode(QString& url);
    static void encode(QString& url);

    operator QString() const;


    virtual QString toString(bool encodedPath = false, bool forcePrependProtocol = true) const;

    virtual bool cdUp();

    static bool isRelativeUrl(const QString &url);

protected:
    virtual void reset();
    virtual bool parse(const QString& url);

private:
    Q3UrlPrivate *d;

};

#endif //QT_NO_URL
#endif
