/****************************************************************************
**
** Copyright (C) 1992-2004 Trolltech AS. All rights reserved.
**
** This file is part of the network module of the Qt Toolkit.
**
** Licensees holding valid Qt Preview licenses may use this file in
** accordance with the Qt Preview License Agreement provided with the
** Software.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
** information about Qt Commercial License Agreements.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef QUDPSOCKET_H
#define QUDPSOCKET_H

#include "qabstractsocket.h"
#include "qhostaddress.h"

#if defined(QT_LICENSE_PROFESSIONAL)
#define QM_EXPORT_NETWORK
#else
#define QM_EXPORT_NETWORK Q_NETWORK_EXPORT
#endif

class QUdpSocketPrivate;

class QM_EXPORT_NETWORK QUdpSocket : public QAbstractSocket
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(QUdpSocket)
public:
    QUdpSocket(QObject *parent = 0);
    virtual ~QUdpSocket();

    bool bind(const QHostAddress &address, Q_UINT16 port);
    bool bind(Q_UINT16 port = 0);

    bool hasPendingDatagrams() const;
    Q_LONGLONG pendingDatagramSize() const;
    Q_LONGLONG readDatagram(char *data, Q_LONGLONG maxlen,
                            QHostAddress *host = 0, Q_UINT16 *port = 0);
    Q_LONGLONG writeDatagram(const char *data, Q_LONGLONG len,
                             const QHostAddress &host, Q_UINT16 port);
    inline Q_LONGLONG writeDatagram(const QByteArray &datagram,
                                    const QHostAddress &host, Q_UINT16 port)
        { return writeDatagram(datagram.data(), datagram.size(), host, port); }

private:
    Q_DISABLE_COPY(QUdpSocket)
};

#endif // QUdpSOCKET_H
